//==== ABOUT =============================================================================//

/*
  This file is a firmware for an Arduino Nano compatible board
  running inside DIGENERATOR, a SYNTHFOX Eurorack syntherizer module
  
  synthfoxmodular.com
  
  NAME     :
  VERSION  :
  AUTHOR   :
  CONTACTS :
  DESCRIPTION
    <insert description here>
  CONTROLS
    FREQUENCY : <insert what control changes what here...>
    CHARACTER :
    OPTION 1  :
    OPTION 2  :
    GATE IN 1 :
    GATE IN 2 :
    GATE IN 3 :
    GATE IN 4 :
  OUTPUTS
    GATE :
    DAC  :
*/










//==== defines ===========================================================================//

//---- put your aliases for stuff here. I already pre-made some so you can program more easily.

// use INGATEX to check if a gate input is on
// e. g. if (INGATE2) {...}
#define INGATE1 (PINC & 0x01)
#define INGATE2 (PINC & 0x02)
#define INGATE3 (PINC & 0x04)
#define INGATE4 (PINC & 0x08)

// use OUTGATE_ON to output +12 on gate out on and OUTGATE_OFF to output a zero. OUTGATE_FLIP to invert current state.
// e.g. if (c == 0) OUTGATE_ON;
#define OUTGATE_ON  (PORTB |=  (1 << PB4))
#define OUTGATE_OFF (PORTB &= ~(1 << PB4))
#define OUTGATE_FLIP (PORTB ^= (1 << PB4))

// use CHARACTER to refer to get a [0..255] value from the character knob and CV sum
// e.g. uint8_t temp = CHARACTER;
#define CHARACTER (analogRead(A6) >> 2)

// use OPTA to check if option A is on, OPTB for option B
// e.g. if (OPT1) {...}
#define OPTA (PINC & 0x10)
#define OPTB (PINC & 0x20)










//==== global variables===================================================================//

//---- variables accessed throughout the entire code go here.

uint16_t // declare your 2-byte long integers here
         rnd = 0xAB;

uint8_t  // declare your byte long integers here. uint8_t = byte = unsigned char.
         counter = 0;



// reverse code by https://forum.arduino.cc/u/tcworld/
const byte reverse[256] PROGMEM = {
  0x00, 0x80, 0x40, 0xC0, 0x20, 0xA0, 0x60, 0xE0, 0x10, 0x90, 0x50, 0xD0, 0x30, 0xB0, 0x70, 0xF0,  
  0x08, 0x88, 0x48, 0xC8, 0x28, 0xA8, 0x68, 0xE8, 0x18, 0x98, 0x58, 0xD8, 0x38, 0xB8, 0x78, 0xF8,  
  0x04, 0x84, 0x44, 0xC4, 0x24, 0xA4, 0x64, 0xE4, 0x14, 0x94, 0x54, 0xD4, 0x34, 0xB4, 0x74, 0xF4,  
  0x0C, 0x8C, 0x4C, 0xCC, 0x2C, 0xAC, 0x6C, 0xEC, 0x1C, 0x9C, 0x5C, 0xDC, 0x3C, 0xBC, 0x7C, 0xFC,  
  0x02, 0x82, 0x42, 0xC2, 0x22, 0xA2, 0x62, 0xE2, 0x12, 0x92, 0x52, 0xD2, 0x32, 0xB2, 0x72, 0xF2,  
  0x0A, 0x8A, 0x4A, 0xCA, 0x2A, 0xAA, 0x6A, 0xEA, 0x1A, 0x9A, 0x5A, 0xDA, 0x3A, 0xBA, 0x7A, 0xFA, 
  0x06, 0x86, 0x46, 0xC6, 0x26, 0xA6, 0x66, 0xE6, 0x16, 0x96, 0x56, 0xD6, 0x36, 0xB6, 0x76, 0xF6,  
  0x0E, 0x8E, 0x4E, 0xCE, 0x2E, 0xAE, 0x6E, 0xEE, 0x1E, 0x9E, 0x5E, 0xDE, 0x3E, 0xBE, 0x7E, 0xFE, 
  0x01, 0x81, 0x41, 0xC1, 0x21, 0xA1, 0x61, 0xE1, 0x11, 0x91, 0x51, 0xD1, 0x31, 0xB1, 0x71, 0xF1, 
  0x09, 0x89, 0x49, 0xC9, 0x29, 0xA9, 0x69, 0xE9, 0x19, 0x99, 0x59, 0xD9, 0x39, 0xB9, 0x79, 0xF9,  
  0x05, 0x85, 0x45, 0xC5, 0x25, 0xA5, 0x65, 0xE5, 0x15, 0x95, 0x55, 0xD5, 0x35, 0xB5, 0x75, 0xF5, 
  0x0D, 0x8D, 0x4D, 0xCD, 0x2D, 0xAD, 0x6D, 0xED, 0x1D, 0x9D, 0x5D, 0xDD, 0x3D, 0xBD, 0x7D, 0xFD, 
  0x03, 0x83, 0x43, 0xC3, 0x23, 0xA3, 0x63, 0xE3, 0x13, 0x93, 0x53, 0xD3, 0x33, 0xB3, 0x73, 0xF3,  
  0x0B, 0x8B, 0x4B, 0xCB, 0x2B, 0xAB, 0x6B, 0xEB, 0x1B, 0x9B, 0x5B, 0xDB, 0x3B, 0xBB, 0x7B, 0xFB, 
  0x07, 0x87, 0x47, 0xC7, 0x27, 0xA7, 0x67, 0xE7, 0x17, 0x97, 0x57, 0xD7, 0x37, 0xB7, 0x77, 0xF7,  
  0x0F, 0x8F, 0x4F, 0xCF, 0x2F, 0xAF, 0x6F, 0xEF, 0x1F, 0x9F, 0x5F, 0xDF, 0x3F, 0xBF, 0x7F, 0xFF 
}; 










//==== auxillary functions ===============================================================//

//---- if you can concatenate a bunch of repeating code pieces into a function - do it




//---- below are some premade helpful functions

// call this to push an 8 bit value to the module DAC output
void dac(unsigned char val)
{
  val = pgm_read_byte(&(reverse[val]));
  PORTB &= 0x00; // clear
  PORTD |= (val) & B11110000;  // 4 upper bits go to PORTD
  PORTB |= (val) & B00001111;  // 4 lower bits go to PORTB
}

// updates the default random (rnd) variable. Calling it multiple times every VCO cycle may slow down the module!!
void rndUpdate()
{
  uint16_t temp = ~(rnd >> 15) ^ (rnd >> 14) ^ ((rnd >> 11) & (rnd >> 7)) + 7;
  rnd = rnd << 1;
  rnd |= temp & 0x01;
}

// "clipping byte subtraction" - if the result is "negative", outputs 0, otherwise outputs the difference
byte subIfNotNeg(byte num, byte toSub)
{
  if (toSub >= num)
    return 0;
   return num - toSub;
}

// "clipping addition" - if the result overflows a byte (>255), returns 255
byte addIfNotOver(byte num, byte toAdd)
{
  if (toAdd >= (255-num))
    return 255;
  return num + toAdd;
}

// quick divide by 3, code by https://stackoverflow.com/users/1202769/qwertz
byte div3(byte num)
{
    byte sum = 0;
    while (num > 3) {
        sum = (num >> 2) + sum;
        num = (num >> 2) + (num & 3);
    }
    if (num == 3)
        sum = sum + 1;
    return sum; 
}










//==== interrupt handlers ================================================================//

//---- clock VCO interrupt
void onVCOCycle() 
{ 
  // put your code that happens on each analog VCO tick here
  
  // these 4 lines generate a sawtooth on DAC out and a suboctave square on GATE out.
  // made so you could test the loading process right off!
  counter += 16;
  dac(counter);
  if (counter == 0)
    OUTGATE_FLIP;
}

//---- gate input interrupt
void onGates()
{
   // put the code that happens when any of the 4 gate inputs goes high here
} 

// uncomment the /**/'s if you want to use the timer interrupt.
// the interrupt is setup in the arduino setup function - uncomment it, too, or the uC will get stuck!
/*
//---- internal timer interrupt
ISR(TIMER0_COMPA_vect)
{
  // code that happens on each timer cycle goes here!
  
}
*/










//==== arduino setup & loop ==============================================================//

//---- this code will be ran once, at the start of arduino's worktime
void setup() {
  
  // in/out registers setup - don't touch unless you know what you're doing
  DDRD = 0xF0; // 0-3 in, 4-7 out
  DDRB = 0x1F; // 0-4 out, 5-7 in
  DDRC = 0x00; // all in
  
  // interrupt setup - also don't touch unless you know what you're doing
  attachInterrupt(digitalPinToInterrupt(2),onGates,RISING); 
  attachInterrupt(digitalPinToInterrupt(3),onVCOCycle,RISING); 
  
  // timer setup - uncomment this AND the timer interrupt handler up there if you wanna use it!
  // also don't touch unless you know what you're doing (which only may be changing the time scale etc)
  /*
  TCCR0A = 0;
  TCCR0B = 0;
  TCNT0  = 0;
  OCR0A = 124;// = (16*10^6) / (2000*64) - 1 (must be <256)
  TCCR0A |= (1 << WGM01);
  TCCR0B |= (1 << CS01) | (1 << CS00);   
  TIMSK0 |= (1 << OCIE0A);
  */
  
  //allow interrupts  - also don't touch unless you know what you're doing
  sei();
  
  
  // ADC speedup  - also don't touch unless you know what you're doing
  #ifndef cbi
  #define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
  #endif
  #ifndef sbi
  #define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
  #endif
  sbi(ADCSRA,ADPS2) ;
  cbi(ADCSRA,ADPS1) ;
  cbi(ADCSRA,ADPS0) ;
  
  //---- your setup code goes here
  
  
}

//---- whatever is in the loop gets executed every step IF the controller isn't busy handling VCO/gate interrupts!
void loop() {
  //---- your loop code goes here
  
}

